/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.service;

import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.common.kit.ConvertKit;
import com.iohao.mmo.common.provide.item.ItemTypeIdConst;
import com.iohao.mmo.equip.entity.*;
import com.iohao.mmo.equip.enums.PropertyNameEnum;
import com.iohao.mmo.equip.kit.EquipRandomKit;
import lombok.AllArgsConstructor;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 装备词条库
 *
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备库实现类
 */
@Service
@AllArgsConstructor
public class EquipEntryTemplateService {

    final MongoTemplate mongoTemplate;

    /**
     * 通过id查询装备词条库
     * @param id 装备词条库id
     * @return
     */
    public EquipEntryTemplate findById(String id) {
        return  mongoTemplate.findById(id, EquipEntryTemplate.class);
    }

    /**
     * 通过code查询装备词条库
     * @param code 装备词条库编码
     * @return
     */
    public EquipEntryTemplate findByCode(String code) {
        return  mongoTemplate.findOne(
                Query.query(new Criteria("code").is(code)), EquipEntryTemplate.class);
    }

    /**
     * 保存装备词条库
     * @param equipEntryTemplate 装备词条库
     */
    public void save(EquipEntryTemplate equipEntryTemplate) {
        mongoTemplate.save(equipEntryTemplate);
    }

    /**
     * 根据装备词条库id列表批量删除装备库
     * @param idList 装备词条库id列表
     */
    public void delBatch(List<String> idList) {
        Query query = new Query(new Criteria("id").in(idList));
        mongoTemplate.remove(query, EquipEntryTemplate.class);
    }

    /**
     * 根据装备部位、数量 随机词条模板
     * @param position 部位
     * @param count 随机数量
     * @return
     */
    public List<EquipEntryTemplate> randomEntryTemplateBatch(String position,int count){
        // 创建条件对象
        Criteria criteria = new Criteria();
        criteria.orOperator(Criteria.where("positionList").isNull(),Criteria.where("positionList").is(position));

        Query query = new Query();
        query.addCriteria(criteria);

        List<EquipEntryTemplate> equipEntryTemplateList = mongoTemplate.find(query,EquipEntryTemplate.class);

        return EquipRandomKit.weightRandom(equipEntryTemplateList,count);
    }

    /**
     * 根据装备等级、部位、数量 随机词条
     * @param level 等级
     * @param position 部位
     * @param count 随机数量
     * @return
     */
    public List<EquipEntry> randomEntryBatch(int level,String position,int count){
        List<EquipEntry> equipEntryList = new ArrayList<>();
        List<EquipEntryTemplate> equipEntryTemplateList = randomEntryTemplateBatch(position,count);
        for(EquipEntryTemplate equipEntryTemplate:equipEntryTemplateList){
            EquipEntry equipEntry = ConvertKit.sourceToTarget(equipEntryTemplate, EquipEntry.class);
            //筛选出符合条件的list
            List<EquipEntryProperty> newEntryPropertyList = filterByStream(equipEntryTemplate.getEntryPropertyList(),level);
            equipEntry.setEntryPropertyList(newEntryPropertyList);
            equipEntryList.add(equipEntry);
        }

        return equipEntryList;
    }

    /**
     * 列表过滤
     * @param equipEntryPropertyList
     * @param level
     * @return
     */
    public List<EquipEntryProperty> filterByStream(List<EquipEntryProperty> equipEntryPropertyList,int level) {
        return equipEntryPropertyList.stream()
                .filter(equipEntryProperty -> equipEntryProperty.getLevelMin()<=level)
                .filter(equipEntryProperty -> equipEntryProperty.getLevelMax()>=level)
                .collect(Collectors.toList());
    }

    /**
     * 根据装备等级、编号查找词条模板并随机出词条
     * @param level 等级
     * @param code 编号
     * @return
     */
    public EquipEntry entryByCode(int level,String code){
        EquipEntryTemplate equipEntryTemplate = findByCode(code);
        EquipEntry equipEntry = ConvertKit.sourceToTarget(equipEntryTemplate, EquipEntry.class);
        //筛选出符合条件的list
        List<EquipEntryProperty> newEntryPropertyList = filterByStream(equipEntryTemplate.getEntryPropertyList(),level);
        equipEntry.setEntryPropertyList(newEntryPropertyList);

        return equipEntry;
    }

}
