package com.iohao.mmo.excel.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.iohao.mmo.excel.dto.DemoExcelDTO;
import com.iohao.mmo.excel.entity.DemoExcel;
import com.iohao.mmo.common.kit.ConvertKit;
import com.iohao.mmo.excel.service.DemoExcelService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.util.ArrayList;
import java.util.List;

/**
 * Excel模板读取类（自定义读取类）
 *
 * @author Toby javatangbin@163.com
 */
public class ExcelDataCustomListener<E, T> extends AnalysisEventListener<T> {
    private static final Logger LOGGER = LoggerFactory.getLogger(ExcelDataCustomListener.class);
    /**
     * 每隔2000条存储数据库，然后清理list，方便内存回收
     */
    private static final int BATCH_COUNT = 2000;
    private final List<DemoExcel> list = new ArrayList<>();

    /**
     * 通过构造器注入Service
     */
    private final DemoExcelService demoExcelService;

    /**
     * 构造方法
     *
     * @param demoExcelService Service对象
     */
    public ExcelDataCustomListener(DemoExcelService demoExcelService) {
        this.demoExcelService = demoExcelService;
    }

    /**
     * 每条数据解析完，都会调用此方法
     */
    @Override
    public void invoke(T data, AnalysisContext context) {
        LOGGER.info("解析到一条数据:{}", JSONObject.toJSONString(data));
        DemoExcelDTO excel = ConvertKit.sourceToTarget(data, DemoExcelDTO.class);

        // 这里对数据做自定义设置
        DemoExcel entity = ConvertKit.sourceToTarget(excel, DemoExcel.class);
        if(entity.getStatus()==0){
            entity.setRemark(entity.getRemark()+"-----我这里对状态为停用的数据做了自定义修改"); //转化为时间戳
        }
        list.add(entity);

        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (list.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            list.clear();
        }
    }
    /**
     * 所有数据解析完成了 都会来调用
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
        LOGGER.info("所有数据解析完成！");
    }
    /**
     * 加上存储数据库
     */
    private void saveData() {
        LOGGER.info("{}条数据，开始存储数据库！", list.size());
        try {
            //根据code批量更新或者插入
            demoExcelService.updateBatch(list);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
        LOGGER.info("存储数据库成功！");
    }
}
